@@ -4,9 +4,6 @@ package    # hide from PAUSE
 use strict;
 use warnings;
 use FindBin qw( $Bin );
-use Gearman::XS qw(:constants);
-use Gearman::XS::Client;
-use Gearman::XS::Server;
 use Gearman::Driver;
 use Net::Telnet;
 
@@ -14,7 +11,23 @@ my ( $host, $port ) = ( '127.0.0.1', 4731 );
 
 $|++;
 
-sub new { return bless {}, shift }
+BEGIN {
+    eval "require Gearman::XS";
+    unless ($@) {
+        eval "require Gearman::XS qw(:constants);";
+        eval "require Gearman::XS::Client;";
+        eval "require Gearman::XS::Server;";
+    }
+    else {
+        eval "require Gearman::Client;";
+        eval "require Gearman::Server;";
+        eval "require Danga::Socket;";
+    }
+}
+
+sub new {
+    return bless {}, shift;
+}
 
 sub run_gearmand {
     my ($self) = @_;
@@ -47,13 +60,36 @@ sub run_gearman_driver {
 }
 
 sub gearman_client {
-    my $client = Gearman::XS::Client->new();
-    $client->add_server( $host, $port );
+    my ( $self, $h, $p ) = @_;
+    $h ||= $host;
+    $p ||= $port;
+
+    my $client;
+    if ( has_xs() ) {
+        $client = Gearman::XS::Client->new();
+        $client->add_server( $h, $p );
+    }
+    else {
+        no strict 'refs';
+        $client = Gearman::Client->new( exceptions => 1 );
+        $client->job_servers("${h}:${p}");
+
+        # Fake Gearman::XS interface
+        *{"Gearman::Client::do"} = sub { my $result = shift->do_task(@_); return ( 0, $result ? $$result : 0 ); };
+        *{"Gearman::Client::do_background"} = sub { shift->dispatch_background(@_); };
+    }
     return $client;
 }
 
-sub gearman_server {
-    return Gearman::XS::Server->new( $host, $port );
+sub gearman_server_run {
+    if ( has_xs() ) {
+        my $server = Gearman::XS::Server->new( $host, $port );
+        $server->run();
+    }
+    else {
+        my $server = Gearman::Server->new( port => $port );
+        Danga::Socket->EventLoop();
+    }
 }
 
 sub gearman_driver {
@@ -76,6 +112,12 @@ sub telnet_client {
     return $telnet;
 }
 
+sub has_xs {
+    eval "require Gearman::XS";
+    return 1 unless $@;
+    return 0;
+}
+
 sub DESTROY {
     my ($self) = @_;
 
